/**
 ** ivmobCntrlTaskCodels.c
 **
 ** Codels used by the control task ivCntrlTask
 **
 ** Author:
 ** Date:
 **
 **/

#include <math.h>
#include <stdio.h>
#include <stdint.h>

#include <portLib.h>
#include <ivmob/ivmob.h>

#include "ivmobConst.h"
#include "server/ivmobHeader.h"

/*----------------------------------------------------------------------
 * controlActivate - control codel of EXEC request Activate
 */
STATUS
controlActivate(int *active, int *report)
{
	IVMOB_STATUS_STR *status = &SDI_F->status;

	/* activate is only valid in active/inactive mode */
	switch (status->operational_mode) {
	case IVMOB_MODE_ACTIVE:
	case IVMOB_MODE_INACTIVE:
		/* activate cannot be sent while in remote mode */
		if (status->rs_mode == IVMOB_MANUAL) {
			*report = S_ivmob_MANUAL_MODE;
			return ERROR;
		}
		return OK;
	case IVMOB_MODE_SAFETY:
		*report = S_ivmob_EMERGENCY_STOP;
		return ERROR;
	case IVMOB_MODE_WAIT_RESET:
		*report = S_ivmob_WAIT_RESET;
		return ERROR;
	default:
		*report = S_ivmob_IVMOBLIB_ERROR;
		return ERROR;
	}
}
/*------------------------------------------------------------------------
 * controlCmdParam  -  control codel of CONTROL request SetCmdParam
 *
 * Description:
 * Report: OK
 *                 S_ivmob_BAD_CMD_PARAMETERS
 *
 * Returns:    OK or ERROR
 */

STATUS
controlCmdParam(GENPOS_CART_CMD_PARAM *cmd, int *report)
{
	/* ... add your code here ... */
	return OK;
}

/*------------------------------------------------------------------------
 * computeStopRef  -  control codel of CONTROL request Stop
 *
 * Description:
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
computeStopRef(int *report)
{
	GENPOS_CART_STATE *robotId;           /* Config robot */
	GENPOS_CART_REF *refId;               /* Consigne */
	double dtArret, dtRetard;  /* Temps prise en compte et temps deceleration */
	double dSArret, dTArret;   /* Longueurs lin et ang de decel */
	double angleFinal;         /* Angle a l'arrivee */
	double period;

	robotId = &ivmobDataStrId->robot;
	refId = &ivmobDataStrId->ref;

	/* Longueur d'arret */
	period = EXEC_TASK_PERIOD(IVMOB_MOTIONTASK_NUM);
	dtArret = MAX (fabs(robotId->v) / IVMOB_DEFAULT_MAXIMUM_ACCEL,
		       fabs(robotId->w) / IVMOB_DEFAULT_MAXIMUM_YAW_ACCEL);
	dtRetard =  1 * period;
	dSArret = robotId->v * (dtArret / 2 + dtRetard);
	dTArret = robotId->w * (dtArret / 2 + dtRetard);
	angleFinal = robotId->theta + dTArret;

	/* Consigne */
	refId->x = robotId->xRob + dSArret * cos (angleFinal);
	refId->y = robotId->yRob + dSArret * sin (angleFinal);
	refId->v = 0. ;
	refId->w = 0. ;
	return(OK);
}

/*------------------------------------------------------------------------
 * trackStart  -  control codel of EXEC request Track
 *
 * Description:    Check tracking mode compatibility and start tracking
 *                 if possible
 * Report: OK
 *                 S_ivmob_POSTER_NOT_FOUND
 *                 S_ivmob_BAD_REF
 *                 S_ivmob_CMD_STOP_TRACK
 *                 S_ivmob_MOTORS_OFF
 *
 * Returns:    OK or ERROR
 */

STATUS
trackStart(IVMOB_TRACK_PARAMS *params, int *report)
{
	IVMOB_STATUS_STR *status = &SDI_F->status;
	int retval = OK;

	/* Check if mode changed */
	switch (status->rs_mode) {
	case IVMOB_EMERGENCY:
		*report = S_ivmob_EMERGENCY_STOP;
		return ERROR;
	case IVMOB_MOTORS_OFF:
		*report = S_ivmob_MOTORS_OFF;
		return ERROR;
	case IVMOB_MANUAL:
		*report = S_ivmob_MANUAL_MODE;
		return ERROR;
	default:
		*report = OK;
		break;
	}
	/* Start correct tracking mode */
	switch (params->mode) {
	case GENPOS_NO_TRACKING:
		status->rs_mode = IVMOB_IDLE;
		ivmobDataStrId->trackStr.trackMode = params->mode;
		*report = S_ivmob_CMD_STOP_TRACK;
		retval = ERROR;
		break;
	case GENPOS_TRACK_POS:
	case GENPOS_TRACK_CONFIG:
	case GENPOS_TRACK_SPEED:
		ivmobDataStrId->trackStr.trackMode = params->mode;
		break;
	default:
		*report = S_ivmob_BAD_REF;
		retval = ERROR;
	}
	return retval;
}


/*------------------------------------------------------------------------
 * GotoConfigCtrl  -  control codel of CONTROL request GotoConfig
 *
 * Description:
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
GotoCtrl(GENPOS_CART_REF *ref, int *report)
{
	GENPOS_CART_REF *refId = &SDI_F->ref;
	IVMOB_STATUS_STR *status = &SDI_F->status;

	/* Check if mode changed */
	switch (status->rs_mode) {
	case IVMOB_EMERGENCY:
		*report = S_ivmob_EMERGENCY_STOP;
		return ERROR;
	case IVMOB_MOTORS_OFF:
		*report = S_ivmob_MOTORS_OFF;
		return ERROR;
	case IVMOB_MANUAL:
		*report = S_ivmob_MANUAL_MODE;
		return ERROR;
	default:
		*report = OK;
		break;
	}
	/* Start correct tracking mode */
	switch (ref->dataType) {
	case GENPOS_POS_AND_SPEED_DATA:
		status->rs_mode = IVMOB_TRACK;
		ivmobDataStrId->trackStr.trackMode = GENPOS_TRACK_CONFIG;
		break;
	case GENPOS_SPEED_DATA:
		status->rs_mode = IVMOB_TRACK;
		ivmobDataStrId->trackStr.trackMode = GENPOS_TRACK_SPEED;
		break;
	default:
		*report = S_ivmob_BAD_REF;
		return ERROR;
	}
	memcpy(refId, ref, sizeof(GENPOS_CART_REF));
	return OK;
}

/*------------------------------------------------------------------------
 * Recale  -  control codel of CONTROL request SetPos
 *
 * Description:
 * Report (bilan): OK
 *                 S_ivmob_TRACKING_ON
 *
 * Return:    OK ou ERROR
 */

STATUS
Recale (IVMOB_CART_3DCONFIG *setPosStr, int *bilan)
{

	GENPOS_CART_STATE *configId;
	IVMOB_CART_3DSTATE *config3dId;

	configId = &ivmobDataStrId->robot;
	config3dId = &ivmobDataStrId->robot3d;
	
	configId->xRob = setPosStr->x;
	configId->yRob = setPosStr->y;
	config3dId->zRob = setPosStr->z;

	configId->theta = setPosStr->theta;

	SDI_F->gyro.gyroToRobotOffset = configId->theta	- SDI_F->gyro.gyroTheta;

	return OK;
}
